import pandas as pd

def read_jdeps_file(path_to_file, fill_dependencies=False):
	'''Imports a Java Dependency Analyzer output

	Command for generating the file set externaly:

		`jdeps -verbose <your_classes_directory> > name_your_file.txt`

	You can then read the `name_your_file.txt` file with this function.

	Parameters:
		- `fill_dependencies`: Lists all target dependencies also on the source side (with empty target and type values).

	The read in output gives you three Series:

		- "from": The full qualified class name of the origin of a dependency relationship
		- "to": The full qualified class name of the target of the dependency relationship
		- "type": "The kind of dependency (different values depending on your own application, good for filtering out some special dependencies)
	'''
	deps = pd.read_csv(
		path_to_file,
		names=["raw"],
		sep="\n")

	# class entries begin with three whitespaces
	deps = deps[deps['raw'].str.startswith("   ")]
	# separates the source from the target
	splitted = deps['raw'].str.split("->", n=1, expand=True)
	# remove whitespaces from source
	deps['from'] = splitted[0].str.strip()
	# get the target and the artifact names
	splitted_2 = splitted[1].str.split(" ", n=2)
	deps['to'] = splitted_2.str[1]
	deps['type'] = splitted_2.str[2].str.strip()

	# some algorithms need to have all target dependencies also on the source side
	if fill_dependencies:
		additional_deps = pd.DataFrame()
		additional_deps['from'] = deps[~deps['to'].isin(deps['from'])]['to'].unique()
		additional_deps['to'] = pd.np.nan
		additional_deps['type'] = pd.np.nan

		deps = pd.concat([deps[['from', 'to', 'type']], additional_deps], ignore_index=True)

	return deps



import pandas as pd
import git
from io import StringIO


def log_numstat(path_to_git_repo):

	meta_data_format_string = "%x09%x09%x09%h%x09%aI%x09%aN"

	# connect to repo
	git_bin = git.Repo(path_to_git_repo).git

	# execute log command
	git_log = git_bin.execute('git log --no-merges --no-renames --numstat --pretty=format:"' + meta_data_format_string + '"')

	# read in the log
	git_log = pd.read_csv(
		StringIO(git_log), 
		sep="\t", 
		header=None,
		names=['additions', 'deletions', 'file', 'sha', 'timestamp', 'author']
	)

	# convert to DataFrame
	commit_data = git_log[['additions', 'deletions', 'file']]\
		.join(git_log[['sha', 'timestamp', 'author']]\
		.fillna(method='ffill'))\
		.dropna()
	
	files = pd.read_csv(StringIO(git_bin.execute('git ls-files')), 
		sep="\n", 
		header=None,
		names=['file'])
		
	commits = commit_data[commit_data['file'].isin(files['file'])].copy()
	commits['timestamp'] = pd.to_datetime(commits['timestamp'])
	return commits


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ozapfdis",
    version="0.0.1",
    author="Markus Harrer",
    author_email="ozapfdis@markusharrer.de",
    description="Open Zippy Analysis Project For Data In Software: A helper library that let's you import all kinds of software related data into the data analysis framework pandas.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/feststelltaste/ozapfdis",
    packages=setuptools.find_packages(),
    install_requires=[
          'pandas',
          'gitpython'
      ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ]
)